﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace TempTest
{
	public partial class Form1 : Form
	{
		private bool loading = true;
		private TEMPerDeviceType[] deviceTypes;
		private TEMPer temper;

		public Form1()
		{
			InitializeComponent();

			int temperHumCount = 0;
			int temperCount = 0;
			deviceTypes = TEMPer.DetectDevices();
			foreach (TEMPerDeviceType deviceType in deviceTypes)
			{
				switch (deviceType)
				{
					case TEMPerDeviceType.TEMPer:
						DeviceList.Items.Add("TEMPer #" + temperCount);
						temperCount++;
						break;
					case TEMPerDeviceType.TEMPerHUM:
						DeviceList.Items.Add("TEMPerHUM #" + temperHumCount);
						temperHumCount++;
						break;
				}
			}

			loading = false;

			if (DeviceList.Items.Count > 0)
				DeviceList.SelectedIndex = 0;
		}

		private void DeviceList_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (!loading)
			{
				if (temper != null)
					temper.Dispose();

				TempCalibrationNum.Enabled = false;
				HumCalibrationNum.Enabled = false;
				SaveCalibrationButton.Enabled = false;
				ReadTimer.Enabled = false;
				InfoText.Text = "";
				ReadingText.Text = "";
				TempCalibrationNum.Value = 0M;
				HumCalibrationNum.Value = 0M;

				try
				{
					TEMPerDeviceType deviceType = deviceTypes[DeviceList.SelectedIndex];
					int deviceNum = 0;
					// Count all devices of the same type before the selected one to determine its index.
					for (int i = DeviceList.SelectedIndex - 1; i >= 0; i--)
					{
						if (deviceTypes[i] == deviceType)
							deviceNum++;
					}

					temper = new TEMPer(deviceType, deviceNum);

					InfoText.Text = temper.DeviceVersion;

					if (temper.SupportsCorrection)
					{
						if (temper.SupportsTemperature)
						{
							TempCalibrationNum.Value = (decimal) temper.TemperatureCorrection;
							TempCalibrationNum.Enabled = true;
						}
						if (temper.SupportsHumidity)
						{
							HumCalibrationNum.Value = (decimal) temper.HumidityCorrection;
							HumCalibrationNum.Enabled = true;
						}
						SaveCalibrationButton.Enabled = true;
					}

					ReadTimer_Tick(null, null);
					ReadTimer.Enabled = true;
				}
				catch (Exception)
				{
					InfoText.Text = "Device error";
				}
			}
		}

		private void ReadTimer_Tick(object sender, EventArgs e)
		{
			if (!ReadBackgroundWorker.IsBusy)
			{
				ReadBackgroundWorker.RunWorkerAsync();
			}
		}

		private void ReadBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
		{
			temper.ReadValues();
		}

		private void ReadBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			string s = "";
			if (temper.SupportsTemperature)
			{
				s += (s.Length > 0 ? "  |  " : "") +
					temper.Temperature.ToString("N2") + " °C";
			}
			if (temper.SupportsHumidity)
			{
				s += (s.Length > 0 ? "  |  " : "") +
					temper.Humidity.ToString("N2") + "%";
			}
			if (temper.SupportsTemperature && temper.SupportsHumidity)
			{
				s += (s.Length > 0 ? "  |  " : "") +
					"dew: " + temper.DewPoint.ToString("N2") + " °C  |  " +
					"abs: " + temper.AbsoluteHumidity.ToString("N2") + " g/m³";
			}
			ReadingText.Text = s;
			IndicateActivity();
		}

		private void SaveCalibrationButton_Click(object sender, EventArgs e)
		{
			if (temper == null || !temper.SupportsCorrection) return;
			
			//ReadTimer.Enabled = false;
			//// Wait for all reading operations to complete.
			//Thread.Sleep(500);
			if (temper.WriteCalibration((double) TempCalibrationNum.Value, (double) HumCalibrationNum.Value))
			{
				MessageBox.Show("OK");
			}
			else
			{
				MessageBox.Show("Failed");
			}
			//ReadTimer.Enabled = true;
		}

		private void IndicateActivity()
		{
			ActivityIndicator.BackColor = Color.ForestGreen;
			ActivityTimer.Start();
		}

		private void ActivityTimer_Tick(object sender, EventArgs e)
		{
			ActivityTimer.Stop();
			ActivityIndicator.BackColor = Color.Empty;
		}
	}
}
